function [Simu] = MH_regression_with_MLL(y,X,nb_MCMC,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Function : Simulation of parameter posterior distributions of simple regression using MH sampler 
%%%%% and computation of MLLs according to different methods
%%%%% (Chib,Importance Sampling,Harmonic Mean)
%%%%% Be careful :  Harmonic mean estimator
%%%%% requires too many draws to converge to the true estimate. Do not use
%%%%% it in practice.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% - post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2
%%%%% under CP priors
%%%%% - beta_ols and sigma_ols are the OLS results.
%%%%% - MLL_Chib : MLL computed from MCMC output
%%%%% - MLL_IS : MLL computed by IS
%%%%% - MLL_HM : MLL computed by Harmonic mean 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<4)
    graph=0;
end

%%%% Check if the variables y and X exhibit an appropriate size.
T = max(size(y));
dimension = min(size(X));
if(dimension~=size(X,2))
    X = X';
end
%%%% Computation of the OLS estimator
beta_ols = (X'*X)\X'*y;
SSR = (y-X*beta_ols)'*(y-X*beta_ols);
sigma_ols = SSR/(T-dimension);
Mat_XX = X'*X;
Vec_Xy = X'*y;

%%%% Set the prior values
%%%%%%
%%% Prior : Conjugate Prior with a Diagonal M_0
%%% sigma^2 ~ IG(a,b) : Note that v_0 = 2a and s_0 = 2b (cfr. slides p112).
%%%%%%
IG_b = 1;%(y-X*beta_ols)'*(y-X*beta_ols)/2; %% IG_2(a,b) = IG(a/2,b/2)
IG_a = 1;%3/2;
var_uninformative = 100; %% We fix the variance of each beta equal to var_uninformative.
beta_0 = zeros(dimension,1);
Sigma_0 = diag(ones(dimension,1)*(var_uninformative));
det_Sigma_0 = det(Sigma_0);
inv_Sigma_0 = diag(ones(dimension,1)*(1/var_uninformative));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Random Walk with adaptation during the burn-in
%%%%%%%%%%% Choosing a small step size at first
%%%%%%%%%%% See Atchad and Rosenthal (2005) for more explanations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
min_max_var = [1e-5 30]; 
eta = 0.6;

%%%%%%%%%%%%%%%%%%%%%%%
%%%% MCMC iterations %%
%%%%%%%%%%%%%%%%%%%%%%%

adapt_rate = ones(dimension+1,1);
accept_rate = zeros(dimension+1,1);
burn_in = ceil(0.3*nb_MCMC);
nb_post = nb_MCMC-burn_in;
beta_post = zeros(dimension,nb_post);
sigma_post = zeros(nb_post,1);
like_post = zeros(nb_post,1);

disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');

%%%%%%%%%%%%%%%
%%%% MCMC starting values
%%%%%%%%%%%%%%%
beta = beta_ols;
sigma = sigma_ols;
iter = 1;
current_like = (-T/2)*log(2*pi*sigma) - (y-X*beta)'*(y-X*beta)/(2*sigma);
current_prior_sigma = IG_a*log(IG_b) - gammaln(IG_a) -(IG_a+1)*log(sigma) - IG_b/sigma;
current_prior_beta = -0.5*dimension*log(2*pi)-0.5*log(det_Sigma_0) - 0.5*(beta-beta_0)'*inv_Sigma_0*(beta-beta_0);
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    %%%%%%%%%%%
    %%%% Saving posterior samples
    %%%%%%%%%%%
    if(i>burn_in)
        beta_post(:,iter) = beta;
        sigma_post(iter) = sigma;
        like_post(iter) = current_like;
        iter = iter +1;
    end
    
    %%%%%%%%%%%%%
    %%% Update of sigma|beta
    %%%%%%%%%%%%%
    sigma_prop = sigma + sqrt(adapt_rate(dimension+1))*randn();
    if(sigma_prop>0)
        like_prop = (-T/2)*log(2*pi*sigma_prop) - (y-X*beta)'*(y-X*beta)/(2*sigma_prop);
        prior_sigma_prop = IG_a*log(IG_b) - gammaln(IG_a) -(IG_a+1)*log(sigma_prop) - IG_b/sigma_prop;
        if(exp(like_prop+prior_sigma_prop-current_like-current_prior_sigma)>rand())
            sigma = sigma_prop;
            current_like = like_prop;
            current_prior_sigma = prior_sigma_prop;
            accept_rate(dimension+1) = accept_rate(dimension+1) +1;
        end
    end
    
    
    
    %%%%%%%%%%%%%
    %%% Update of beta|Sigma
    %%%%%%%%%%%%%
    beta_prop = beta;
    for q=1:dimension
        beta_prop(q) = beta(q) + sqrt(adapt_rate(q))*randn();
        like_prop = (-T/2)*log(2*pi*sigma) - (y-X*beta_prop)'*(y-X*beta_prop)/(2*sigma);
        prior_beta_prop = -0.5*dimension*log(2*pi)-0.5*log(det_Sigma_0) - 0.5*(beta_prop-beta_0)'*inv_Sigma_0*(beta_prop-beta_0);
        if(exp(like_prop+prior_beta_prop-current_like-current_prior_beta)>rand())
            beta = beta_prop;
            current_like = like_prop;
            current_prior_beta = prior_beta_prop;
            accept_rate(q) = accept_rate(q) +1;
        else
           beta_prop(q) = beta(q); 
        end
    end
    
    for q=1:dimension+1
        adapt_rate(q) = max(min_max_var(1),adapt_rate(q) + (accept_rate(q)/i - 0.45)/(i^eta));
        if(adapt_rate(q)>min_max_var(2))
            adapt_rate(q) = min_max_var(2);
        end
    end

end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.accept_rate = accept_rate/nb_MCMC;
Simu.adapt_rate = adapt_rate;
Simu.post_beta = beta_post;
Simu.post_sigma = sigma_post;
Simu.beta_ols = beta_ols;
Simu.sigma_ols = sigma_ols;
Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
Simu.post_var_beta = cov(beta_post');
Simu.post_mean_sigma = mean(sigma_post,2);
Simu.post_std_sigma = std(sigma_post,0,2);
Simu.post_like = like_post;
maxi = max(like_post);
Simu.MLL_HM = maxi -log(mean(1./exp(like_post-maxi)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%    CHIB's MLL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('**************************************');
disp('***** BEGINNING MLL COMPUTATION ******');
disp('*********** CHIB FORMULA *************');
disp('**************************************');
%%%%%%%%%%%%%
%%% Computation of the MLL from the MCMC output (using Chib's formulas).
%%%%%%%%%%%%%
[likelihood, ind] = max(like_post);
beta_star = beta_post(:,ind);
sigma_star = sigma_post(ind);
log_dens_prior = IG_a*log(IG_b) - gammaln(IG_a) -(IG_a+1)*log(sigma_star) - IG_b/sigma_star -0.5*dimension*log(2*pi)-0.5*log(det(Sigma_0)) - 0.5*(beta_star-beta_0)'*inv_Sigma_0*(beta_star-beta_0);

%%%%%%%%%%%%%
%%% Decomposition of the posterior distribution into two known densities :
%%% f(sigma*,beta*|Y) = f(sigma*|Y,beta*) f(beta*|Y)
%%% f(beta*|Y) = integral f(beta*|Y,sigma) f(sigma|Y) dsigma
%%%%%%%%%%%%%
IG_post_a = T/2 + IG_a;
IG_post_b = IG_b + 0.5*sum((y-X*beta_star).^2);
log_post_sigma = IG_post_a*log(IG_post_b) - gammaln(IG_post_a) -(IG_post_a+1)*log(sigma_star) - IG_post_b/sigma_star;

post_beta_dens = zeros(nb_post,1);

for q=1:nb_post
    progressbar(q/nb_post);
    inv_sigma = 1/sigma_post(q);
    Sigma_cond = inv(inv_sigma*Mat_XX + inv_Sigma_0);
    mu_cond = Sigma_cond*(inv_sigma*Vec_Xy + inv_Sigma_0*beta_0);
    post_beta_dens(q) = exp(-0.5*dimension*log(2*pi) -0.5*log(det(Sigma_cond)) - 0.5*((beta_star-mu_cond)'*inv(Sigma_cond)*(beta_star-mu_cond)));
end

%%%%%%%%%%%
%%%% Computation of the MLL according to the Bayes theorem
%%%%%%%%%%%
Simu.MLL_Chib = likelihood + log_dens_prior - log_post_sigma - log(mean(post_beta_dens));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%   MLL By Importance Sampling %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('**************************************');
disp('***** BEGINNING MLL COMPUTATION ******');
disp('******** Importance Sampling *********');
disp('**************************************');
%%%%%%%%%%%%%%%
%%%% Proposal distribution for beta ~ N(beta_ols,10*Sigma_cond*sigma)
%%%%%%%%%%%%%%%
%%%% Proposal distribution for sigma ~ IG(T/4,SSR/4)
%%%%%%%%%%%%%%%
nb_IS = 1e4;
log_proposal_sigma = zeros(nb_IS,1);
log_proposal_beta = zeros(nb_IS,1);
log_likelihood = zeros(nb_IS,1);
log_prior = zeros(nb_IS,1);

IG_prop_a = T/4;
IG_prop_b = SSR/4;
Sigma_prop = cov(beta_post');
mu_prop = mean(beta_post,2);
det_Sigma_prop = det(Sigma_prop);
inv_Sigma_prop = inv(Sigma_prop);
for q=1:nb_IS
   progressbar(q/nb_IS);
   sigma = 1/gamrnd(IG_prop_a,1/IG_prop_b); 
   beta = mvnrnd(mu_prop,Sigma_prop)';
   
   log_likelihood(q) = (-T/2)*log(2*pi*sigma) - (y-X*beta)'*(y-X*beta)/(2*sigma);
   log_prior(q) = IG_a*log(IG_b) - gammaln(IG_a) -(IG_a+1)*log(sigma) - IG_b/sigma -0.5*dimension*log(2*pi)-0.5*log(det(Sigma_0)) - 0.5*(beta-beta_0)'*inv_Sigma_0*(beta-beta_0);
   log_proposal_sigma(q) = IG_prop_a*log(IG_prop_b) - gammaln(IG_prop_a) -(IG_prop_a+1)*log(sigma) - IG_prop_b/sigma;
   log_proposal_beta(q) = -0.5*dimension*log(2*pi) -0.5*log(det_Sigma_prop) - 0.5*((beta-mu_prop)'*inv_Sigma_prop*(beta-mu_prop));
end
log_computation = log_likelihood+log_prior-log_proposal_beta-log_proposal_sigma;
[maxi] = max(log_computation);
dens_ratio = exp(log_computation-maxi);
Simu.MLL_IS = log(mean(dens_ratio)) + maxi;




%%%%%%%%%%%%%
%%% Testing for the convergence by Geweke's test, Cusum plot and ACF
%%%%%%%%%%%%%
Simu.test_Geweke = Geweke_test(beta_post);
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1)
    figure;
    for z=1:dimension
        subplot(ceil(0.5*dimension),2,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    autocorr(sigma_post,200),title('Sample ACF of \sigma^2');
end