function [Simu] = Gibbs_regression_Carlin_Gelman_Smith(y,X,nb_MCMC,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Function : Estimation of a CP model with 2 regimes using the method 
%%%%% of Carlin, Gelman and Smith. The Marginal log-likelihood is not
%%%%% computed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% - post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2
%%%%% under CP priors
%%%%% - beta_ols and sigma_ols are the OLS results of the starting values.
%%%%% - post_tau : MCMC draws of the break date sampled by Griddy-Gibbs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<4)
    graph=0;
end

%%%% Check if the variables y and X exhibit an appropriate size.
T = max(size(y));
dimension = min(size(X));
if(dimension~=size(X,2))
    X = X';
end
tau = round(T/4 + rand()*T*0.5);

%%%% Computation of the OLS estimator
beta_ols = zeros(dimension,2);
SSR = zeros(2,1);
sigma_ols = zeros(2,1);
beta_ols(:,1) = (X(1:tau,:)'*X(1:tau,:))\X(1:tau,:)'*y(1:tau);
beta_ols(:,2) = (X(tau+1:end,:)'*X(tau+1:end,:))\X(tau+1:end,:)'*y(tau+1:end);
SSR(1) = (y(1:tau)-X(1:tau,:)*beta_ols(:,1))'*(y(1:tau)-X(1:tau,:)*beta_ols(:,1));
SSR(2) = (y(tau+1:end)-X(tau+1:end,:)*beta_ols(:,2))'*(y(tau+1:end)-X(tau+1:end,:)*beta_ols(:,2));
sigma_ols(1) = SSR(1)/(tau);
sigma_ols(2) = SSR(2)/(T-tau);




%%%% Set the prior values
%%%%%%
%%% Prior : Conjugate Prior with a Diagonal M_0
%%% sigma^2 ~ IG(a,b) 
%%%%%%
IG_b = 2;
IG_a = 3;
var_uninformative = 100; %% We fix the variance of each beta equal to var_uninformative.
beta_0 = zeros(dimension,1);
%Sigma_0 = diag(ones(dimension,1)*(var_uninformative));
inv_Sigma_0 = diag(ones(dimension,1)*(1/var_uninformative));



%%%%%%%%%%%
%%%% MCMC Initialisation
%%%%%%%%%%%
burn_in = ceil(0.3*nb_MCMC);
nb_post = nb_MCMC-burn_in;
beta_post = zeros(2*dimension,nb_post);
tau_post = zeros(nb_post,1);
sigma_post = zeros(2,nb_post);
like_post = zeros(nb_post,1);

disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');
log_cond_tau = zeros(T-1,1);
%%%%%%%%%%%%%%%
%%%% MCMC starting values
%%%%%%%%%%%%%%%
beta = beta_ols;
sigma = sigma_ols;
inv_sigma = 1./sigma_ols;
iter = 1;
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    
    
    %%%%%%%%%%%%%
    %%% Update of sigma1|beta,tau
    %%%%%%%%%%%%%
    IG_post_a_MCMC = tau/2 + IG_a;
    IG_post_b_MCMC = IG_b + 0.5*sum((y(1:tau)-X(1:tau,:)*beta(:,1)).^2);
    inv_sigma(1) = gamrnd(IG_post_a_MCMC,1/IG_post_b_MCMC);
    sigma(1) = 1/inv_sigma(1);
    
    %%%%%%%%%%%%%
    %%% Update of sigma2|beta,tau
    %%%%%%%%%%%%%
    IG_post_a_MCMC = (T-tau)/2 + IG_a;
    IG_post_b_MCMC = IG_b + 0.5*sum((y(tau+1:end)-X(tau+1:end,:)*beta(:,2)).^2);
    inv_sigma(2) = gamrnd(IG_post_a_MCMC,1/IG_post_b_MCMC);
    sigma(2) = 1/inv_sigma(2);
    
    
    %%%%%%%%%%%%%
    %%% Update of beta1|Sigma,tau
    %%%%%%%%%%%%%
    Mat_XX = X(1:tau,:)'*X(1:tau,:);
    Vec_Xy = X(1:tau,:)'*y(1:tau);
    Sigma_cond = inv(inv_sigma(1)*Mat_XX + inv_Sigma_0);
    mu_cond = Sigma_cond*(inv_sigma(1)*Vec_Xy + inv_Sigma_0*beta_0);
    beta(:,1) = mvnrnd(mu_cond,Sigma_cond)';
    
    %%%%%%%%%%%%%
    %%% Update of beta2|Sigma,tau
    %%%%%%%%%%%%%
    Mat_XX = X(tau+1:end,:)'*X(tau+1:end,:);
    Vec_Xy = X(tau+1:end,:)'*y(tau+1:end);
    Sigma_cond = inv(inv_sigma(2)*Mat_XX + inv_Sigma_0);
    mu_cond = Sigma_cond*(inv_sigma(2)*Vec_Xy + inv_Sigma_0*beta_0);
    beta(:,2) = mvnrnd(mu_cond,Sigma_cond)';
    
    
    %%%%%%%%%%%%%
    %%% Update of tau|Sigma,beta
    %%%%%%%%%%%%%
    Mat_XX = X(tau+1:end,:)'*X(tau+1:end,:);
    Vec_Xy = X(tau+1:end,:)'*y(tau+1:end);
    Sigma_cond = inv(inv_sigma(2)*Mat_XX + inv_Sigma_0);
    mu_cond = Sigma_cond*(inv_sigma(2)*Vec_Xy + inv_Sigma_0*beta_0);
    beta(:,2) = mvnrnd(mu_cond,Sigma_cond)';
    
    for t=1:T-1
        log_cond_tau(t) = (-t/2)*log(2*pi*sigma(1)) - sum((y(1:t)-X(1:t,:)*beta(:,1)).^2)/(2*sigma(1)) + (-(T-t)/2)*log(2*pi*sigma(2)) - sum((y(t+1:end)-X(t+1:end,:)*beta(:,2)).^2)/(2*sigma(2));
    end
    [maxi] = max(log_cond_tau);
    cond_tau = exp(log_cond_tau-maxi);
    cond_tau = cumsum(cond_tau/sum(cond_tau));
    tau = find(cond_tau>rand(),1);
    if(isempty(tau))
        tau = T-1;
    end
    current_log_dens = log_cond_tau(tau);
    
    %%%%%%%%%%%
    %%%% Saving posterior samples
    %%%%%%%%%%%
    if(i>burn_in)
        beta_post(:,iter) = reshape(beta,dimension*2,1);
        sigma_post(:,iter) = sigma;
        tau_post(iter) = tau;
        like_post(iter) = current_log_dens;
        iter = iter +1;
    end

end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.post_beta = beta_post;
Simu.post_sigma = sigma_post;
Simu.post_tau = tau_post;
Simu.beta_ols = beta_ols;
Simu.sigma_ols = sigma_ols;
Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
Simu.post_var_beta = cov(beta_post');
Simu.post_mean_sigma = mean(sigma_post,2);
Simu.post_std_sigma = std(sigma_post,0,2);
Simu.post_like = like_post;





%%%%%%%%%%%%%
%%% Testing for the convergence by Geweke's test, Cusum plot and ACF
%%%%%%%%%%%%%
Simu.test_Geweke = Geweke_test(beta_post);
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1)
    figure;
    for z=1:dimension*2
        subplot(ceil(dimension),2,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    subplot(2,1,1),autocorr(sigma_post(1,:),200),title('Sample ACF of \sigma_1^2');
    subplot(2,1,2),autocorr(sigma_post(2,:),200),title('Sample ACF of \sigma_2^2');
end