function [Simu MLL_struct] = Gibbs_regression_chib(y,X,regime,nb_MCMC,graph,MLL_computation,MLL_struct)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% function that simulates posterior distribution of a CP model with y as dependent variable and X as explanatory variables
%%%%% The simulation is carried out by MCMC using the Chib algorithm
%%%%% (latent variable + forward-backward algorithm)
%%%%% The Maginal likelihood is computed by MCMC using the local formula
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% regime is the number of regimes that we want in the model
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%% MLL_computation = 1 (default) if you want to estimate the MLL
%%%%% MLL_struct stands for a structure created and used by the program. Do
%%%%% not specify any value for it.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% - post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2
%%%%% under CP priors
%%%%% - post_tau : MCMC posterior draws of the break dates
%%%%% - post_trans_mat : posterior draw of the diagonal of the transition
%%%%% matrix
%%%%% - post_like : the posterior densities of the drawn parameters
%%%%% - MLL : the marginal likelihood of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% EXAMPLE FOR RuNNING THE PROGRAM :
%%%%%%%% [Simu MLL_struct] = Gibbs_regression_chib(y,X,2,1000);
%%%%% Run a CP-model with two regimes simulated with 1000 MCMC iterations
%%%%% No graphics assessing the MCMC convergence is given
%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% [Simu MLL_struct] = Gibbs_regression_chib(y,X,4,1000,1);
%%%%% Run a CP-model with four regimes simulated with 1000 MCMC iterations
%%%%% Graphics assessing the MCMC convergence are given.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%% By default, set the variable graph = 0.
if(nargin<7)
    if(nargin<6)
        MLL_computation = 1;
        if(nargin<5)
            graph=0;
        end
    end
    Aux_Gibbs = 0;
else
    if(isfield(MLL_struct,'beta_star'))
        MLL_struct.Aux_Gibbs = MLL_struct.Aux_Gibbs +1;
        Aux_Gibbs = MLL_struct.Aux_Gibbs;
    else
        warning('MLL struct should not be used as function argument'); %#ok<*WNTAG>
        Aux_Gibbs = 0;
    end 
end


%%%% Check if the variables y and X exhibit an appropriate size.
T = max(size(y));
dimension = min(size(X));
if(dimension~=size(X,2))
    X = X';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Set the Hyper-parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Prior : 
%%% beta_i ~ N(beta_0,var_uninformative)
%%% sigma^-2 ~ G(a,b) 
%%% p_ii ~ Beta(alpha,beta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var_uninformative = 1000; %% We fix the variance of each beta equal to var_uninformative.
beta_0 = zeros(dimension,1);
inv_Sigma_0 = diag(ones(dimension,1)*(1/var_uninformative));
det_inv_Sigma_0 = det(inv_Sigma_0);

%%% hyper-parameters for the variances
IG_b = 100; 
IG_a = 0.01;

%%% hyper-parameters for each probability of staying in the same regime
alpha_p = 0.5;
beta_p = 0.5;

%%% constants for easing the computation of densities in the MCMC.
cst_gam = -IG_a*log(IG_b) -gammaln(IG_a);
cst_beta = gammaln(alpha_p+beta_p)-gammaln(alpha_p)-gammaln(beta_p);

%%%%%%%%%%%
%%%% MCMC Initialisation
%%%%%%%%%%%
burn_in = ceil(0.3*nb_MCMC);
if(Aux_Gibbs == 0)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Search for MCMC starting values
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('**********************************');
    disp('***** MCMC starting values *******');
    disp('**********************************');
    nb_init = round(T);
    init_log_like = zeros(nb_init,1);
    %%% We randomly draw many state vectors and for each one, we compute
    %%% the conrresponding OLS estimates as well as the implied density
    %%% The chosen starting values are those exhibiting the highest
    %%% likelihood
    [init_sn init_tau] = initialisation_sn(nb_init,T,regime,1); 
    for q=1:nb_init
        [~,~, init_log_like(q)] = OLS_estimator(y,X,init_sn(:,q));
    end
    [~, ind] = max(init_log_like);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% MCMC starting values
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    sn = init_sn(:,ind);
    tau = init_tau(:,ind);
    
%     tau = [121   156   260   384   421   502   664]';
%     [sn] = trans_sn(tau,T);
    [beta,sigma] = OLS_estimator(y,X,sn);
    inv_sigma = 1./sigma;
    trans_mat = tau./(tau+1);
    trans_mat(end) = 1;
    
    

else
    beta = MLL_struct.beta_star;
    sigma = MLL_struct.sigma_star;
    inv_sigma = 1./sigma;
    trans_mat = MLL_struct.trans_mat_star;
    sn = MLL_struct.sn_star;
    [tau] = trans_sn_into_cass(sn,T);
    nb_MCMC = 1000;
    burn_in = 0;
end
beta_star = beta;
sigma_star = sigma;

nb_post = nb_MCMC-burn_in;
beta_post = zeros(regime*dimension,nb_post);
tau_post = zeros(regime,nb_post);
trans_mat_post = zeros(regime,nb_post);
sigma_post = zeros(regime,nb_post);
like_post = zeros(nb_post,1);
log_conditional_posterior_densities = zeros(2,nb_post);

iter = 1;
disp('************************************');
disp('***** Beginning of the MCMC  *******');
disp('************************************');
for i=1:nb_MCMC
    %progressbar(i/nb_MCMC);
    
    %%%%%%%%%%%%%
    %%% Update of sigma|Y_T,beta,sn
    %%%%%%%%%%%%%
    IG_post_a_MCMC = tau(1)/2 + IG_a;
    IG_post_b_MCMC = 1/(1/IG_b + 0.5*sum((y(1:tau(1))-X(1:tau(1),:)*beta(:,1)).^2));
    
    if(Aux_Gibbs<2)
        inv_sigma(1) = gamrnd(IG_post_a_MCMC,IG_post_b_MCMC);
        sigma(1) = 1/inv_sigma(1);
    end
    log_post_dens_sigma =  -gammaln(IG_post_a_MCMC) - IG_post_a_MCMC*log(IG_post_b_MCMC) -(IG_post_a_MCMC-1)*log(sigma_star(1)) -1/(IG_post_b_MCMC*sigma_star(1));
    for k=2:regime
        IG_post_a_MCMC = (tau(k)-tau(k-1))/2 + IG_a;
        IG_post_b_MCMC = 1/(1/IG_b + 0.5*sum((y(tau(k-1)+1:tau(k))-X(tau(k-1)+1:tau(k),:)*beta(:,k)).^2));
        
        if(Aux_Gibbs<2)
            inv_sigma(k) = gamrnd(IG_post_a_MCMC,IG_post_b_MCMC);
            sigma(k) = 1/inv_sigma(k);
        end
        log_post_dens_sigma =  log_post_dens_sigma -gammaln(IG_post_a_MCMC) - IG_post_a_MCMC*log(IG_post_b_MCMC) -(IG_post_a_MCMC-1)*log(sigma_star(k)) -1/(IG_post_b_MCMC*sigma_star(k));
    end


    %%%%%%%%%%%%%
    %%% Update of beta|Y_T,Sigma,sn
    %%%%%%%%%%%%%
    
    Mat_XX = X(1:tau(1),:)'*X(1:tau(1),:);
    Vec_Xy = X(1:tau(1),:)'*y(1:tau(1));
    Sigma_cond = inv(Mat_XX + inv_Sigma_0*sigma(1));
    mu_cond = Sigma_cond*(Vec_Xy + inv_Sigma_0*beta_0*sigma(1));
    Sigma_cond = sigma(1)*Sigma_cond;
    [Triangular,test_Sig] = cholcov(Sigma_cond);
    if(test_Sig==0)
        beta(:,1) = mu_cond + Triangular'*randn(dimension,1); %mvnrnd(mu_cond,Sigma_cond)';
    end
    log_post_dens_beta = -0.5*dimension*log(2*pi) -0.5*log(det(Sigma_cond)) -0.5*(beta_star(:,1)-mu_cond)'*inv(Sigma_cond)*(beta_star(:,1)-mu_cond);
    for k=2:regime
        Mat_XX = X(tau(k-1)+1:tau(k),:)'*X(tau(k-1)+1:tau(k),:);
        Vec_Xy = X(tau(k-1)+1:tau(k),:)'*y(tau(k-1)+1:tau(k));
        Sigma_cond = inv(Mat_XX + inv_Sigma_0*sigma(k));
        mu_cond = Sigma_cond*(Vec_Xy + inv_Sigma_0*beta_0*sigma(k));
        Sigma_cond = sigma(k)*Sigma_cond;
        [Triangular,test_Sig] = cholcov(Sigma_cond);
        if(test_Sig==0)
            beta(:,k) = mu_cond + Triangular'*randn(dimension,1);%mvnrnd(mu_cond,Sigma_cond)';
        end
        log_post_dens_beta = log_post_dens_beta -0.5*dimension*log(2*pi) -0.5*log(det(Sigma_cond)) -0.5*(beta_star(:,k)-mu_cond)'*inv(Sigma_cond)*(beta_star(:,k)-mu_cond);
    end
    
    
    %%%%%%%%%%%%%
    %%% Update of sn|Y_T,Sigma,beta
    %%%%%%%%%%%%%
    if(regime>1)
        [sn current_log_dens] = Forward_Backward(y,X,beta,sigma,trans_mat);
        [tau] = trans_sn_into_cass(sn,T);
    else
        current_log_dens = -0.5*T*log(2*pi*sigma) - sum((y-X*beta).^2)/(2*sigma);
    end
    
    %%%%%%%%%%%%%
    %%% Update of P|Y_T,sn
    %%%%%%%%%%%%%
    if(Aux_Gibbs<1 && regime>1)
        trans_mat(1) = betarnd(alpha_p+tau(1)-1,beta_p+1);
        for k=2:regime-1
            trans_mat(k) = betarnd(alpha_p+(tau(k)-tau(k-1)-1),beta_p+1);
        end
    end
    
    


    
    %%%%%%%%%%%
    %%%% Saving posterior samples
    %%%%%%%%%%%
    if(i>burn_in)
        beta_post(:,iter) = reshape(beta,dimension*regime,1);
        sigma_post(:,iter) = sigma;
        tau_post(:,iter) = tau;
        trans_mat_post(:,iter) = trans_mat;
        like_post(iter) = current_log_dens;
        log_conditional_posterior_densities(:,iter) = [log_post_dens_sigma log_post_dens_beta]';
        iter = iter +1;
    end

end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.post_beta = beta_post;
Simu.post_sigma = sigma_post;
Simu.post_tau = tau_post;
Simu.post_P = trans_mat_post;
Simu.post_mean_tau = mean(tau_post,2);
Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
Simu.post_var_beta = cov(beta_post');
Simu.post_mean_sigma = mean(sigma_post,2);
Simu.post_std_sigma = std(sigma_post,0,2);
Simu.post_var_sigma = cov(sigma_post');
Simu.post_like = like_post;
Simu.log_cond_post_dens = log_conditional_posterior_densities;





%%%%%%%%%%%%%
%%% Testing for the convergence by Geweke's test, Cusum plot and ACF
%%%%%%%%%%%%%
Simu.test_Geweke = Geweke_test(beta_post);
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1 && Aux_Gibbs == 0)
    figure;
    for z=1:dimension*regime
        subplot(ceil(dimension),regime,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    for z=1:regime
        subplot(1,regime,z),autocorr(sigma_post(z,:),200),title(['Sample ACF of \sigma^2 : ' num2str(z)]);
    end
end

if(MLL_computation==1)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% MLL by Chib  %%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if(Aux_Gibbs == 0)
        disp('******************************************');
        disp('***** MLL by Chib (local formula)  *******');
        disp('******************************************');
        
        [log_like_at_mode ind] = max(Simu.post_like);
        beta_star = Simu.post_beta(:,ind);
        sigma_star = Simu.post_sigma(:,ind);
        trans_mat_star = Simu.post_P(:,ind);
        tau_star = Simu.post_tau(:,ind);
        [sn_star] = trans_sn(Simu.post_tau(:,ind),T);
        
        disp('*************************************************');
        disp('***** Computation of the prior densities  *******');
        disp('*************************************************');
        log_prior_beta = -0.5*dimension*log(2*pi) + 0.5*log(det_inv_Sigma_0) -0.5*(beta_star(1:dimension)-beta_0)'*inv_Sigma_0*(beta_star(1:dimension)-beta_0);
        log_prior_sigma = cst_gam -(IG_a-1)*log(sigma_star(1)) -1/(IG_b*sigma_star(1));
        
        if(regime>1)
            log_prior_trans_mat = cst_beta + (alpha_p-1)*log(trans_mat_star(1)) + (beta_p-1)*log(1-trans_mat_star(1));
        else
            log_prior_trans_mat = 0;
        end
        
        for k=2:regime
            log_prior_beta = log_prior_beta -0.5*dimension*log(2*pi) + 0.5*log(det_inv_Sigma_0) -0.5*(beta_star((k-1)*dimension+1:k*dimension)-beta_0)'*inv_Sigma_0*(beta_star((k-1)*dimension+1:k*dimension)-beta_0);
            log_prior_sigma = log_prior_sigma + cst_gam -(IG_a-1)*log(sigma_star(k)) -1/(IG_b*sigma_star(k));%cst_gam -(IG_a+1)*log(sigma_star(k)) -IG_b/sigma_star(k);
            if(k<regime)
                log_prior_trans_mat = log_prior_trans_mat + cst_beta + (alpha_p-1)*log(trans_mat_star(k)) + (beta_p-1)*log(1-trans_mat_star(k));
            end
        end
        
        disp('*****************************************************');
        disp('***** Computation of the posterior densities  *******');
        disp('*****************************************************');
        post_like_trans_mat = 0;
        if(regime>1)
            nb_MCMC_post_draw = max(size(like_post));
            log_post_like_trans_mat = zeros(nb_MCMC_post_draw,1);
            post_beta_p = beta_p*ones(regime-1,1)+1;
            for i=1:nb_MCMC_post_draw
                post_alpha_p = max(alpha_p+diff([1;tau_post(1:regime-1,i)])-1,1);
                log_post_like_trans_mat(i) = sum(gammaln(post_alpha_p+post_beta_p)-gammaln(post_alpha_p)-gammaln(post_beta_p) + (post_alpha_p).*log(trans_mat_star(1:regime-1)) + (post_beta_p).*log(1-trans_mat_star(1:regime-1)));
            end
            maxi = max(log_post_like_trans_mat);
            post_like_trans_mat = log(mean(exp(log_post_like_trans_mat-maxi))) + maxi;
        else
            nb_MCMC_post_draw = max(size(like_post));
            log_post_dens_sigma = zeros(nb_MCMC_post_draw,1);
            for i=1:nb_MCMC_post_draw
                beta = beta_post(:,i);
                IG_post_a_MCMC = T/2 + IG_a;
                IG_post_b_MCMC = 1/(1/IG_b + 0.5*sum((y-X*beta).^2));
                log_post_dens_sigma(i) =  -gammaln(IG_post_a_MCMC) - IG_post_a_MCMC*log(IG_post_b_MCMC) -(IG_post_a_MCMC-1)*log(sigma_star(1)) -1/(IG_post_b_MCMC*sigma_star(1)); %cst_gam -(IG_post_a_MCMC+1)*log(sigma_star(1)) -IG_post_b_MCMC/sigma_star(1);
            end
            maxi = max(log_post_dens_sigma);
            post_dens_sigma = log(mean(exp(log_post_dens_sigma-maxi))) + maxi;
            
            Mat_XX = X'*X;
            Vec_Xy = X'*y;
            Sigma_cond = inv(Mat_XX/sigma_star + inv_Sigma_0);
            mu_cond = Sigma_cond*(Vec_Xy/sigma_star + inv_Sigma_0*beta_0);
            log_post_dens_beta = -0.5*dimension*log(2*pi) -0.5*log(det(Sigma_cond)) -0.5*(beta_star-mu_cond)'*inv(Sigma_cond)*(beta_star-mu_cond);
            
        end
        
        MLL_struct.beta_star = reshape(beta_star,dimension,regime);
        MLL_struct.sigma_star = sigma_star;
        MLL_struct.trans_mat_star = trans_mat_star;
        MLL_struct.sn_star = sn_star;
        MLL_struct.tau_star = tau_star;
        
        MLL_struct.log_like_at_mode = log_like_at_mode;
        MLL_struct.log_prior_sigma = log_prior_sigma;
        MLL_struct.log_prior_beta = log_prior_beta;
        MLL_struct.log_prior_p = log_prior_trans_mat;
        MLL_struct.log_post_dens_p = post_like_trans_mat;
        MLL_struct.Aux_Gibbs = 0;
        
        if(regime>1)
            [~, MLL_struct] = Gibbs_regression_chib(y,X,regime,nb_MCMC,graph,MLL_computation,MLL_struct);
        else
            MLL_struct.post_like_sigma = post_dens_sigma;
            MLL_struct.post_like_beta = log_post_dens_beta;
            MLL_struct.MLL =  log_like_at_mode + log_prior_sigma + log_prior_beta - log_post_dens_beta - post_dens_sigma;
        end
        Simu.MLL = MLL_struct.MLL;
    elseif(Aux_Gibbs == 1)
        maxi = max(log_conditional_posterior_densities(1,:));
        MLL_struct.post_like_sigma = log(mean(exp(log_conditional_posterior_densities(1,:)-maxi))) + maxi;
        
        [~, MLL_struct] = Gibbs_regression_chib(y,X,regime,nb_MCMC,graph,MLL_computation,MLL_struct);
    else
        maxi = max(log_conditional_posterior_densities(2,:));
        MLL_struct.post_like_beta = log(mean(exp(log_conditional_posterior_densities(2,:)-maxi))) + maxi;
        MLL_struct.MLL = MLL_struct.log_like_at_mode + MLL_struct.log_prior_sigma + MLL_struct.log_prior_beta + MLL_struct.log_prior_p - MLL_struct.log_post_dens_p - MLL_struct.post_like_sigma - MLL_struct.post_like_beta;
        
    end
else
    MLL_struct = 0;
end