function [Simu] = MCMC_GARCH_RW(y,nb_MCMC,RW_step,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Function : Posterior Distribution of the parameters of a GARCH model
%%%%% by adaptive MH MCMC.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the time series (T * 1)
%%%%% nb_MCMC : number of MCMC iterations
%%%%% RW_step : is a scalar. Identical Variance of the Random Walk step for all the GARCH parameters  (variance of the proposal
%%%%% distribution) 
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% post_theta_RW :  MCMC Posterior draws of Omega,Alpha,Beta
%%%%% post_theta_RW_adapt : MCMC posterior draws of Omega,Alpha,Beta with
%%%%% an adaptive RW (during the burn-in)
%%%%% post_like_RW : Log-likelihood of the posterior draws.
%%%%% post_like_RW_adapt : Log-likelihood of the posterior draws obtained
%%%%% from an adaptive RW.
%%%%% The structure also includes the diagnostic tests.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<4)    
    graph = 0;
    if(nargin<3)
        RW_step = 1;
        if(nargin<2)
            nb_MCMC = 5000;
        end
    end
end

%%%% Size of the time series saved in the variable T.
T = max(size(y));
if(T~=size(y,1))
    y = y';
end
y_sq = y.*y;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Looking for an initial value %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[theta like_current] = starting_values(y_sq);
theta_RW_adapt = theta;
like_current_RW_adapt = like_current;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Bounds of the priors : Omega ~ U[0,bound_omega] ; alpha ~ U[0,1] ;
%%%%%%%%%% beta|alpha ~ U[0,1-alpha] %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bound_omega = 10;




burn_in = ceil(0.3*nb_MCMC);
post_like = zeros(nb_MCMC,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Random Walk with adaptation during the burn-in
%%%%%%%%%%% Choosing a small step size at first
%%%%%%%%%%% See Atchad and Rosenthal (2005) for more explanations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
RW_step_adapt = [0.005 0.005 0.005]';
RW_theta = zeros(3,nb_MCMC);
RW_adapt_theta = zeros(3,nb_MCMC);
min_max_var = [1e-5 10]; 
eta = 0.6;

%%%%%%%%%%%%%%%%%%%%%%%
%%%% MCMC iterations %%
%%%%%%%%%%%%%%%%%%%%%%%
accept = 1;
accept_adapt = ones(3,1);
for i=1:nb_MCMC
    progressbar(i/nb_MCMC)
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Without adaptation %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    theta_prop = theta;
    for q=1:3
        theta_prop(q) = theta(q) + sqrt(RW_step)*randn();
        test = 0;
        if(q==1)
            if(theta_prop(q)>0 && theta_prop(q)<bound_omega)
                test = 1;
            end
        else
            if(theta_prop(q)>0 && sum(theta_prop(2:3))<1 && theta_prop(q)<1)
                test = 1;
            end
        end
        if(test==1)
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%% Computation of the log-likelihood
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           [log_like_move] = GARCH_likelihood(y_sq,theta_prop); 
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%% Computation of the prior densities
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           log_prior_stay = -log(bound_omega) - log(1-theta(2));
           log_prior_move = -log(bound_omega) - log(1-theta_prop(2));
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%% Computation of the Metropolis ratio
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           if(exp(log_like_move+log_prior_move-like_current-log_prior_stay)>rand())
               like_current = log_like_move;
               theta(q) = theta_prop(q);
               accept = accept+1;
           else
               theta_prop(q) = theta(q);
           end
        else
            theta_prop(q) = theta(q);
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% With adaptation  during the MCMC %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    theta_prop = theta_RW_adapt;
    for q=1:3
        theta_prop(q) = theta_RW_adapt(q) + sqrt(RW_step_adapt(q))*randn();
        test = 0;
        if(q==1)
            if(theta_prop(q)>0 && theta_prop(q)<bound_omega)
                test = 1;
            end
        else
            if(theta_prop(q)>0 && sum(theta_prop(2:3))<1 && theta_prop(q)<1)
                test = 1;
            end
        end
        if(test==1)
           [log_like_move] = GARCH_likelihood(y_sq,theta_prop); 
           log_prior_stay = -log(bound_omega) - log(1-theta_RW_adapt(2));
           log_prior_move = -log(bound_omega) - log(1-theta_prop(2));
           
           if(exp(log_like_move+log_prior_move-like_current_RW_adapt-log_prior_stay)>rand())
               like_current_RW_adapt = log_like_move;
               theta_RW_adapt(q) = theta_prop(q);
               accept_adapt(q) = accept_adapt(q)+1;
           else
               theta_prop(q) = theta_RW_adapt(q);
           end
        else
            theta_prop(q) = theta_RW_adapt(q);
        end
        RW_step_adapt(q) = max(min_max_var(1),RW_step_adapt(q) + (accept_adapt(q)/i - 0.44)/(i^eta));
        if(RW_step_adapt(q)>min_max_var(2))
            RW_step_adapt(q) = min_max_var(2);
        end
            
    end

    
    post_like(i,1) = like_current;
    post_like(i,2) = like_current_RW_adapt;
    RW_adapt_theta(:,i) = theta_RW_adapt;
    RW_theta(:,i) = theta;
end


%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.accept_rate = accept/(3*nb_MCMC);
Simu.accept_rate_adapt = accept_adapt/(nb_MCMC);
Simu.post_theta_RW = RW_theta(:,burn_in+1:end);
Simu.post_theta_RW_adapt = RW_adapt_theta(:,burn_in+1:end);
Simu.post_like_RW = post_like(burn_in+1:end,1);
Simu.post_like_RW_adapt = post_like(burn_in+1:end,2);
Simu.post_step_RW_adapt = RW_step_adapt;
%%%%%%%%%%%%%
%%% Testing the convergence by cusum plot and ACF
%%%%%%%%%%%%%
[Simu.cusum] = cusum_plot(Simu.post_theta_RW,graph);
if(graph==1)
    figure;
    for z=1:3
        subplot(2,3,z),autocorr(Simu.post_theta_RW(z,:),200),title(['Sample ACF of RW  : \theta : ' num2str(z)]);
        subplot(2,3,z+3),autocorr(Simu.post_theta_RW_adapt(z,:),200),title(['Sample ACF of Adapt RW : \theta : ' num2str(z)]);
    end
    figure;
    subplot(2,2,1),plot(Simu.post_like_RW),title('Densities of posterior draws of the RW');
    subplot(2,2,3),plot(Simu.post_like_RW_adapt),title('Densities of posterior draws of the adapted RW');
    subplot(2,2,2),plot(Simu.post_theta_RW'),title('Posterior draws of the RW'),legend('\omega','\alpha','\beta');
    subplot(2,2,4),plot(Simu.post_theta_RW_adapt'),title('Posterior draws of the adapted RW'),legend('\omega','\alpha','\beta');
    
end